<?php
/**
 * print_r converter - convert print_r() to php variable code
 *
 * Author: hakre <hakre.wordpress.com>
 * Copyright (c) 2011, some rights reserved
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program. If not, see <http://www.gnu.org/licenses/>.
 *
 * known:
 *   - string values over multiple lines not supported.
 *   - limited support for objects, stdClass only.
 *
 * best php codepad in town:
 * >> http://codepad.viper-7.com/ <<
 *   +++ php 5.3 support
 *    +++ working regex
 *     +++ html and source view
 *
 * CHANGES:
 * 0.1.1 - allow more whitespace in array-open.
 *       - remove , at the end of values.
 * 0.1.0 - version 0.1.0, fixed some minor issues.
 * 0.0.9 - support for stdClass objects.
 * 0.0.8 - form was closed too early, fixed.
 * 0.0.7 - textarea for output.
 *       - clear / undo clear.
 * 0.0.6 - deal with empty values in parser via state.
 * 0.0.5 - button tooltips.
 *       - input sanitization now upfront.
 *       - html and css updates.
 *       - change output variable-name from $var to $array
 * 0.0.3 - github link opened in frameset eventually, fixed.
 * 0.0.2 - tokenizer less-strict whitespace dealing for array open and close
 *       - cache last input value into cookie
 *       - typo in tokenizer class name, fixed
 *
 * @author hakre
 * @license GPL v3+
 * @version 0.1.1
 * @date 2011-07-24
 */

header('Content-Type: text/html; charset=utf-8');

/* input: either post or cached in cookie */
$i = isset($_POST['i'])? (string)$_POST['i']:'';
if (($ilen = strlen($i)) > $imaxlen = 4096)
    die(sprintf('Maximum input length (post) of %d exceeded: %d.', $imaxlen, $ilen));
$requestHasCookieData = false;
if (!empty($i))
{
    setcookie('i', $i);
    $requestHasCookieData = true;
} elseif (!empty($_COOKIE['i'])) {
    $i = $_COOKIE['i'];
    if (($ilen = strlen($i)) > $imaxlen)
        die(sprintf('Maximum input (cookie) length of %d exceeded: %d.', $imaxlen, $ilen));
    else
        $requestHasCookieData = true;
}

isset($_POST['c']) && $i = '';
$canUndo = $requestHasCookieData && isset($_POST['c']);

/**
 * print_r regex Tokenizer
 */
class PrintrTokenizer implements Iterator
{
    private $tokens = array(
        'array-open' => 'Array\s*\(\s?$',
        'object-open' => 'stdClass Object\s*\($',
        'key' => '\s*\[[^\]]+\]',
        'map' => ' => ',
        'array-close' => '\s*\)\s?$',
        'value' => '(?<= => )[^\n]*$'
    );
    private $buffer;
    private $offset;
    private $index;
    private $current;
    public function __construct($buffer)
    {
        $this->buffer = $buffer;
    }
    private function match($def, $at)
    {
        $found = preg_match(
            "~$def~im", $this->buffer, $match, PREG_OFFSET_CAPTURE, $at
        );
        if (false === $found) die('Regex error.');

        $return = 0;
        if ($found && $at === $match[0][1])
            $return = strlen($match[0][0]);

        return $return;
    }
    private function matchLargest($at)
    {
        $match = $max = 0;
        foreach($this->tokens as $name => $def)
        {
            ($len = $this->match($def, $at))
            && $len > $max
            && ($max = $len)
            && ($match = $name);
        }
        return $match ? array($match, $at, $max) : null;
    }
    public function current()
    {
        return $this->current;
    }
    public function key()
    {
        return $this->index;
    }
    public function next()
    {
        $current = $this->matchLargest($this->offset);
        ($current)
            && ($current = array_merge($current, array(substr($this->buffer, $this->offset, $current[2]))))
            && ($this->offset += $current[2])
            ;
        $this->current = $current;
        $this->index++;
    }
    public function valid()
    {
        return !(null === $this->current);
    }
    public function rewind()
    {
        $this->offset = 0;
        $this->next();
        $this->index = 0;
    }
}

/**
 * print_r Parser
 */
function PrintrParser($buffer) {
    $result = null;
    $rP = &$result;
    $rS = array();
    $level = 0;
    $len = strlen($buffer);
    $offset = 0;
    $tokens = new PrintrTokenizer($buffer);
    $state = 0; // 1: map
    foreach($tokens as $index => $tokenData) {
        list($token, $offset, $length, $text) = $tokenData;
        switch($token)
        {
            case 'array-open':
                $rP = array();
                $state = 0;
                break;
            case 'object-open':
                $rP = new stdClass();
                $state = 0;
                break;
            case 'key':
                if (1 === $state) { // empty value
                    $rP = '';
                    $rSi = count($rS)-1;
                    $rP = &$rS[$rSi];
                    unset($rS[$rSi]);
                    $state = 0;
                }
                $key = preg_replace('~\[(.*)\]~', '$1', trim($text));
                ((string)(int)$key === $key) && $key = (int)$key;
                if (is_object($rP))
                    $rP->$key = null;
                else
                    $rP[$key] = null;
                break;
            case 'map':
                $rS[count($rS)] = &$rP;
                if (is_object($rP))
                    $rP = &$rP->$key;
                else
                    $rP = &$rP[$key];
                $state = 1;
                break;
            case 'value':
                if (is_string($text) && ($text = rtrim($text)) && ',' === substr($text,-1))
                    $text = substr($text, 0, -1);
                ((string)(int)$text === $text) && $text = (int)$text;
                $rP = $text;
                # fall-through intended
            case 'array-close':
                $rSi = count($rS)-1;
                $rP = &$rS[$rSi];
                unset($rS[$rSi]);
                $state = 0;
                break;
        }
    }
    return $result;
}

?>